/*----------------------------------------------------------------------------*/
/*                                                                            */
/* Copyright (c) 1995, 2004 IBM Corporation. All rights reserved.             */
/* Copyright (c) 2005-2009 Rexx Language Association. All rights reserved.    */
/*                                                                            */
/* This program and the accompanying materials are made available under       */
/* the terms of the Common Public License v1.0 which accompanies this         */
/* distribution. A copy is also available at the following address:           */
/* http://www.ibm.com/developerworks/oss/CPLv1.0.htm                          */
/*                                                                            */
/* Redistribution and use in source and binary forms, with or                 */
/* without modification, are permitted provided that the following            */
/* conditions are met:                                                        */
/*                                                                            */
/* Redistributions of source code must retain the above copyright             */
/* notice, this list of conditions and the following disclaimer.              */
/* Redistributions in binary form must reproduce the above copyright          */
/* notice, this list of conditions and the following disclaimer in            */
/* the documentation and/or other materials provided with the distribution.   */
/*                                                                            */
/* Neither the name of Rexx Language Association nor the names                */
/* of its contributors may be used to endorse or promote products             */
/* derived from this software without specific prior written permission.      */
/*                                                                            */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS        */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS          */
/* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   */
/* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,      */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED   */
/* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,        */
/* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY     */
/* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING    */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               */
/*                                                                            */
/*----------------------------------------------------------------------------*/

#ifndef LocalRegistrationManager_HPP_INCLUDED
#define LocalRegistrationManager_HPP_INCLUDED

#include "RegistrationTable.hpp"
#include "LocalAPISubsystem.hpp"
#include "ServiceMessage.hpp"
#include "rexx.h"


// local instance of the registration API...this is a proxy that communicates with the
// server that manages the api tables.
class LocalRegistrationManager : public LocalAPISubsystem
{
public:
    LocalRegistrationManager();

    RexxReturnCode registerCallback(RegistrationType type, const char *name, const char *module,
        const char *proc, const char *userData, bool drop);

    RexxReturnCode registerCallback(RegistrationType type, const char *name, REXXPFN entryPoint,
        const char *userData);

    RexxReturnCode dropCallback(RegistrationType type, const char *name, const char *module);

    RexxReturnCode queryCallback(RegistrationType type, const char *name);

    RexxReturnCode queryCallback(RegistrationType type, const char *name, const char *module, char *userData);

    RexxReturnCode resolveCallback(RegistrationType type, const char *name, const char *module, REXXPFN &entryPoint);
    RexxReturnCode mapReturnResult(ServiceMessage &m);
    virtual RexxReturnCode processServiceException(ServiceException *e);

protected:

    RegistrationTable &locateTable(RegistrationType type);

    bool              locked;            // indicates we've locked the library against unloading
    RegistrationTable functions;         // our tables for process-specific registrations
    RegistrationTable exits;
    RegistrationTable commandHandlers;
};

#endif
